package build

import (
	"strconv"

	"git.akyoto.dev/cli/q/src/build/asm"
	"git.akyoto.dev/cli/q/src/build/expression"
	"git.akyoto.dev/cli/q/src/errors"
)

// CompileAssignment compiles an assignment.
func (f *Function) CompileAssignment(expr *expression.Expression) error {
	name := expr.Children[0].Token.Text()
	register := f.Variables[name].Register

	switch expr.Token.Text() {
	case "=":
		f.ExpressionToRegister(expr.Children[1], register)

	case "+=":
		number, _ := strconv.Atoi(expr.Children[1].Token.Text())
		f.Assembler.RegisterNumber(asm.ADD, register, number)

	case "-=":
		number, _ := strconv.Atoi(expr.Children[1].Token.Text())
		f.Assembler.RegisterNumber(asm.SUB, register, number)

	case "*=":
		number, _ := strconv.Atoi(expr.Children[1].Token.Text())
		f.Assembler.RegisterNumber(asm.MUL, register, number)

	case "/=":
		number, _ := strconv.Atoi(expr.Children[1].Token.Text())
		f.Assembler.RegisterNumber(asm.DIV, register, number)

	default:
		return errors.New(&errors.InvalidInstruction{Instruction: expr.Token.Text()}, f.File, expr.Token.Position)
	}

	return nil
}
